@@ -1,10 +1,16 @@
 # coding=utf-8
 import os, sys, codecs
 from http.server import HTTPServer, SimpleHTTPRequestHandler
+from outdated import check_outdated
 from socketserver import ThreadingMixIn
 import ssl
 from OpenSSL import crypto
 import site
+import requests
+from bs4 import BeautifulSoup
+from urllib.parse import urljoin
+from urllib3.exceptions import InsecureRequestWarning
+
 
 class ThreadingSimpleServer(ThreadingMixIn, HTTPServer):
     pass
@@ -12,8 +18,8 @@ class ThreadingSimpleServer(ThreadingMixIn, HTTPServer):
 class HTTPS:
 
     def __init__(self,host,port,keyfile,certfile):
-        print('keyfile=',keyfile)
-        print('certfile=',certfile)
+        print('keyfile =',keyfile)
+        print('certfile =',certfile)
         context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         context.load_cert_chain(certfile=certfile, keyfile=keyfile)
         httpd = ThreadingSimpleServer((host, port), SimpleHTTPRequestHandler)
@@ -33,9 +39,8 @@ class HTTP:
 class GenCert:
 
     def __init__(self):
-        pwd = self.get_sitepackages_dir()
-        pwd = os.path.join(pwd, 'myhttps')
-        self.certdir = os.path.join(pwd, 'cert')
+        sitepackages_dir = self.get_sitepackages_dir()
+        self.certdir = os.path.join(sitepackages_dir, 'cert')
         self.KEY_FILE = os.path.join(self.certdir, 'key.pem')
         self.CERT_FILE = os.path.join(self.certdir, 'cert.pem')
         self.CERT_FILE = os.path.join(self.certdir, 'cert.pem')
@@ -87,8 +92,10 @@ class GenCert:
     def get_sitepackages_dir(self):
         getsitepackages = site.getsitepackages()
         for i in getsitepackages:
-            if 'site-packages' in i:
-                return i
+            # print(os.path.join(i, 'myhttps'))
+            # print(os.path.isdir(os.path.join(i, 'myhttps')))
+            if os.path.isdir(os.path.join(i, 'myhttps')):
+                return os.path.join(i, 'myhttps')
 
 
 class Functions:
@@ -96,12 +103,9 @@ class Functions:
         pass
 
     def getVersion(self):
-        for line in self.read("__init__.py").splitlines():
-            print(line)
-            if line.startswith('__version__'):
-                delim = '"' if '"' in line else "'"
-                return line.split(delim)[1]
-            # raise RuntimeError("Unable to find version string.")
+        firstline = self.read("__init__.py").splitlines()[0]
+        ver = firstline.split("'")[1]
+        return ver
 
     def getUsage(self):
         st = False
@@ -123,7 +127,66 @@ class Functions:
         with codecs.open(os.path.join(here, rel_path), 'r') as fp:
             return fp.read()
 
+class DownThemAll:
+    def __init__(self):
+
+        pass
+
+
+    def download_wget(self, url, output_dir=None): # not compatible with Windows
+        if output_dir is None:
+            cmd = f'wget -c -r -np -k -L -p --no-check-certificate {url}'
+        else:
+            self.mkdir(output_dir,force=True)
+            cmd = f'wget -c -r -np -k -L -p --no-check-certificate {url} -P {output_dir}'
+        os.system(cmd)
+
+    def download_website(self, url, output_dir='myhttps_download'):
+        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
+
+        response = requests.get(url, verify=False)
+        response.raise_for_status()
+
+        soup = BeautifulSoup(response.text, 'html.parser')
+        links = soup.find_all('a')
+
+        os.makedirs(output_dir, exist_ok=True)
+
+        for link in links:
+            file_name = link.text.strip()
+            if not file_name or file_name == "../":
+                continue
+
+            file_url = urljoin(url, link['href'])
+            save_path = os.path.join(output_dir, file_name)
+
+            if file_name.endswith('/'):
+                print(f"Entering directory: {file_name}")
+                self.download_website(file_url, save_path)
+            else:
+                print(f"Downloading {file_name} from {file_url}")
+                file_response = requests.get(file_url, stream=True, verify=False)
+                file_response.raise_for_status()
+                with open(save_path, 'wb') as f:
+                    for chunk in file_response.iter_content(chunk_size=8192):
+                        f.write(chunk)
+                print(f"Saved: {save_path}")
+
+
+    def mkdir(self, dir, force=False):
+        if not os.path.isdir(dir):
+            if force == True:
+                os.makedirs(dir)
+            else:
+                os.mkdir(dir)
+
+
 def main():
+    ver = Functions().getVersion()
+    print('current version:',ver)
+    is_outdated, latest = check_outdated("myhttps", ver)
+    if is_outdated:
+        print("The package myhttps is out of date. Your version is %s, the latest is %s." % (ver, latest))
     host = '0.0.0.0'
     port = 11443
 
@@ -136,8 +199,18 @@ def main():
         print(usage)
         exit()
     if "--v" in sys.argv:
-        print("pyhttps version: ", version)
+        print("myhttps version: ", version)
         exit()
+
+    if "-url" in sys.argv:
+        url = sys.argv[sys.argv.index("-url") + 1]
+        if "-outdir" in sys.argv:
+            outdir = sys.argv[sys.argv.index("-outdir") + 1]
+            DownThemAll().download_website(url,outdir)
+        else:
+            DownThemAll().download_website(url)
+        exit()
+
     if "-p" in sys.argv:
         port = sys.argv[sys.argv.index("-p") + 1]
         port = int(port)
@@ -149,6 +222,10 @@ def main():
         keyfile = sys.argv[sys.argv.index("-k") + 1]
     if "-mode" in sys.argv:
         mode = sys.argv[sys.argv.index("-mode") + 1]
+
+
+    pwd = os.getcwd()
+    print('current shared dir:',pwd)
     if mode == 'HTTPS':
         _GenCert = GenCert()
         keyfile = _GenCert.KEY_FILE
@@ -166,4 +243,8 @@ def main():
 if __name__ == "__main__":
     # main()
     # GenCert()
+    # Functions().getVersion()
+    # url = 'https://127.0.0.1:11443/'
+    # Functions().
+    # DownThemAll().download_website(url)
     pass
